"""Ezviz API."""
import hashlib
import json
import logging
from random import randint
from uuid import uuid4

import requests
from pyezviz.camera import EzvizCamera
from pyezviz.constants import DefenseModeType, DeviceCatagories

API_BASE_TLD = ".ezvizlife.com"
API_ENDPOINT_AUTH = "/doLogin"
API_ENDPOINT_CLOUDDEVICES = "/api/cloud/v2/cloudDevices/getAll"
API_ENDPOINT_PAGELIST = "/v3/userdevices/v1/devices/pagelist"
API_ENDPOINT_DEVICES = "/v3/devices/"
API_ENDPOINT_SWITCH_STATUS = "/switchStatus"
API_ENDPOINT_PTZCONTROL = "/ptzControl"
API_ENDPOINT_ALARM_SOUND = "/alarm/sound"
API_ENDPOINT_SET_DEFENCE = "/camera/cameraAction!enableDefence.action"
API_ENDPOINT_DETECTION_SENSIBILITY = "/api/device/configAlgorithm"
API_ENDPOINT_DETECTION_SENSIBILITY_GET = "/api/device/queryAlgorithmConfig"
API_ENDPOINT_CAMERA_INFO_GET = "/camera/cameraAction!findAllDevices.action"
API_ENDPOINT_ALARMINFO_GET = "/alarmlog/alarmLogAction!findAlarmLogs.action"
API_ENDPOINT_CHECKLOGIN = "/user/user/userAction!checkLoginInfo.action"
API_ENDPOINT_SET_DEFENCE_SCHEDULE = "/camera/cameraAction!modifyTimerDefence.action"
API_ENDPOINT_SWITCH_DEFENCE_MODE = "/v3/userdevices/v1/group/switchDefenceMode"

DEFAULT_TIMEOUT = 10
MAX_RETRIES = 3


class PyEzvizError(Exception):
    """Handle exception."""


class EzvizClient:
    """Initialize api client object."""

    def __init__(
        self,
        account,
        password,
        region="eu",
        timeout=DEFAULT_TIMEOUT,
    ):
        """Initialize the client object."""
        self.account = account
        self.password = password
        self._session = None
        self._timeout = timeout
        self._csrf_token = 0
        self.api_domain = "i" + region
        self.auth_domain = region + "auth"

    def _login(self):
        """Login to Ezviz' API."""

        # Ezviz API sends md5 of password
        temp = hashlib.md5()
        temp.update(self.password.encode("utf-8"))
        md5pass = temp.hexdigest()
        randommath = randint(1000000000000000000, 9999999999999999999)
        payload = {
            "account": self.account,
            "password": md5pass,
            "from": "4e4148ba90184a7cbd81",
            "r": randommath,
            "returnUrl": "plugin",
            "host": self.api_domain + API_BASE_TLD,
        }

        try:
            req = self._session.post(
                "https://" + self.auth_domain + API_BASE_TLD + API_ENDPOINT_AUTH,
                allow_redirects=False,
                data=payload,
                timeout=self._timeout,
            )

            json_result = req.json()

            if json_result["retcode"] == "1001":
                raise PyEzvizError("Incorrect login details")

            if json_result["retcode"] == "1002":
                raise PyEzvizError("Login error: Captcha required")

            if json_result["retcode"] == "1005":
                raise PyEzvizError("Login error: Incorrect Captcha code")

            # Update cookie with JS Session ID that is generated by website.

            req = self._session.get(
                "https://" + self.api_domain + API_BASE_TLD + "/check_plugin.jsp",
                timeout=self._timeout,
            )

            self._csrf_token = self._session.cookies.get(
                "AS_SessionID", domain=self.api_domain + API_BASE_TLD
            )

        except OSError as err:
            raise PyEzvizError("Can not login to API") from err

        if req.status_code != 200:
            raise PyEzvizError(
                f"Login error: Please check your username/password: {req.text} "
            )

        return True

    def _get_pagelist(self, page_filter=None, json_key=None, max_retries=0):
        """Get data from pagelist API."""

        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        if page_filter is None:
            raise PyEzvizError("Trying to call get_pagelist without filter")

        try:
            req = self._session.get(
                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_PAGELIST,
                params={"filter": page_filter},
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401:
            # session is wrong, need to relogin
            self.login()
            logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
            return self._get_pagelist(page_filter, json_key, max_retries + 1)

        if req.text == "":
            raise PyEzvizError("No data")

        try:
            json_output = req.json()
        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        if json_output["meta"]["code"] != 200:
            # session is wrong, need to relogin
            self.login()
            logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
            return self._get_pagelist(page_filter, json_key, max_retries + 1)

        if json_key is None:
            json_result = json_output
        else:
            json_result = json_output[json_key]

        if not json_result:
            raise PyEzvizError(
                f"Impossible to load the devices, here is the returned response: {req.text} "
            )

        return json_result

    def _get_deviceinfo(self, serial=None, max_retries=0):
        """Get data from a device info API."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        try:
            req = self._session.get(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_CAMERA_INFO_GET,
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401 or req.status_code == 302:
            # session is wrong, need to relogin
            self.login()
            logging.info("Got 302, relogging (max retries: %s)", str(max_retries))
            return self._get_deviceinfo(serial, max_retries + 1)

        if req.text == "":
            raise PyEzvizError("No data")

        try:
            json_output = req.json()

        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        json_result = {}

        for device in json_output["devices"]:
            json_result[device["subSerial"]] = {}
            json_result[device["subSerial"]] = device

        if not json_result:
            raise PyEzvizError(
                f"Impossible to load the devices, here is the returned response: {req.text} "
            )

        if serial:
            return json_result.get(serial)

        return json_result

    def get_alarminfo(self, serial, page_start=0, page_size=1, max_retries=0):
        """Get data from alarm info API."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        try:
            req = self._session.post(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_ALARMINFO_GET,
                data={
                    "objectName": serial,
                    "alarmType": -1,
                    "queryType": 3,
                    "checkState": 2,
                    "pageStart": page_start,
                    "pageSize": page_size,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401 or req.status_code == 302:
            # session is wrong, need to relogin
            self.login()
            logging.info(
                "Got 302 or 401, relogging (max retries: %s)", str(max_retries)
            )
            return self.get_alarminfo(serial, page_start, page_size, max_retries + 1)

        if req.text == "":
            raise PyEzvizError("No data")

        try:
            json_output = req.json()
        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        return json_output

    def _switch_status(self, serial, status_type, enable, max_retries=0):
        """Switch status on a device."""

        try:
            req = self._session.put(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_DEVICES
                + serial
                + "/1/1/"
                + str(status_type)
                + API_ENDPOINT_SWITCH_STATUS,
                data={
                    "enable": enable,
                    "serial": serial,
                    "channelNo": "1",
                    "type": status_type,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401 or req.status_code == 302:
            # session is wrong, need to relogin
            self.login()
            logging.info(
                "Got 302 or 401, relogging (max retries: %s)", str(max_retries)
            )
            return self._switch_status(serial, status_type, enable, max_retries + 1)

        try:
            json_output = req.json()

        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        if json_output["resultCode"] != "0":
            raise PyEzvizError(
                f"Could not set the switch, maybe a permission issue ?: Got {req.status_code} : {req.text})"
            )

        return True

    def load_cameras(self):
        """Load and return all cameras objects."""

        devices = self.get_all_device_infos()
        cameras = []
        supported_categories = [
            DeviceCatagories.COMMON_DEVICE_CATEGORY.value,
            DeviceCatagories.CAMERA_DEVICE_CATEGORY.value,
            DeviceCatagories.BATTERY_CAMERA_DEVICE_CATEGORY.value,
            DeviceCatagories.DOORBELL_DEVICE_CATEGORY.value,
            DeviceCatagories.BASE_STATION_DEVICE_CATEGORY.value,
        ]

        for device in devices:
            if (
                devices.get(device).get("deviceInfos").get("deviceCategory")
                in supported_categories
            ):
                # Add support for connected HikVision cameras
                if devices.get(device).get("deviceInfos").get(
                    "deviceCategory"
                ) == DeviceCatagories.COMMON_DEVICE_CATEGORY.value and not devices.get(
                    device
                ).get(
                    "deviceInfos"
                ).get(
                    "hik"
                ):
                    continue

                # Create camera object

                camera = EzvizCamera(self, device, devices.get(device))

                camera.load()
                cameras.append(camera.status())

        return cameras

    def get_all_device_infos(self):
        """Load all devices and build dict per device serial"""

        deviceinfo = self._get_deviceinfo()
        devices = self.get_page_list()
        result = {}

        for idx, device in enumerate(devices["deviceInfos"]):
            result[device["deviceSerial"]] = {}
            result[device["deviceSerial"]]["deviceInfos"] = device
            result[device["deviceSerial"]]["connectionInfos"] = devices.get(
                "connectionInfos"
            ).get(device["deviceSerial"])
            result[device["deviceSerial"]]["p2pInfos"] = devices.get("p2pInfos").get(
                device["deviceSerial"]
            )
            result[device["deviceSerial"]]["alarmNodisturbInfos"] = devices.get(
                "alarmNodisturbInfos"
            ).get(device["deviceSerial"])
            result[device["deviceSerial"]]["kmsInfos"] = devices.get("kmsInfos").get(
                device["deviceSerial"]
            )
            result[device["deviceSerial"]]["timePlanInfos"] = devices.get(
                "timePlanInfos"
            ).get(device["deviceSerial"])
            result[device["deviceSerial"]]["statusInfos"] = devices.get(
                "statusInfos"
            ).get(device["deviceSerial"])
            result[device["deviceSerial"]]["wifiInfos"] = devices.get("wifiInfos").get(
                device["deviceSerial"]
            )
            result[device["deviceSerial"]]["switchStatusInfos"] = devices.get(
                "switchStatusInfos"
            ).get(device["deviceSerial"])
            result[device["deviceSerial"]]["deviceinfo"] = deviceinfo.get(
                device["deviceSerial"]
            )
            for item in devices["cameraInfos"]:
                if item["deviceSerial"] == device["deviceSerial"]:
                    result[device["deviceSerial"]]["cameraInfos"] = item
                    result[device["deviceSerial"]]["cloudInfos"] = devices.get(
                        "cloudInfos"
                    ).get(item["cameraId"])

        return result

    def get_all_per_serial_infos(self, serial=None):
        """Load all devices and build dict per device serial"""

        if serial is None:
            raise PyEzvizError("Need serial number for this query")

        deviceinfo = self._get_deviceinfo(serial)
        devices = self.get_page_list()
        result = {serial: {}}

        for idx, device in enumerate(devices["deviceInfos"]):
            if device["deviceSerial"] == serial:
                result[device["deviceSerial"]]["deviceInfos"] = device
                result[device["deviceSerial"]]["deviceInfos"] = device
                result[device["deviceSerial"]]["connectionInfos"] = devices.get(
                    "connectionInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["p2pInfos"] = devices.get(
                    "p2pInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["alarmNodisturbInfos"] = devices.get(
                    "alarmNodisturbInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["kmsInfos"] = devices.get(
                    "kmsInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["timePlanInfos"] = devices.get(
                    "timePlanInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["statusInfos"] = devices.get(
                    "statusInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["wifiInfos"] = devices.get(
                    "wifiInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["switchStatusInfos"] = devices.get(
                    "switchStatusInfos"
                ).get(device["deviceSerial"])
                result[device["deviceSerial"]]["deviceinfo"] = deviceinfo
                for item in devices["cameraInfos"]:
                    if item["deviceSerial"] == device["deviceSerial"]:
                        result[device["deviceSerial"]]["cameraInfos"] = item
                        result[device["deviceSerial"]]["cloudInfos"] = devices.get(
                            "cloudInfos"
                        ).get(item["cameraId"])

        return result.get(serial)

    def ptz_control(self, command, serial, action, speed=5):
        """PTZ Control by API."""
        if command is None:
            raise PyEzvizError("Trying to call ptzControl without command")
        if action is None:
            raise PyEzvizError("Trying to call ptzControl without action")

        try:
            req = self._session.put(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_DEVICES
                + serial
                + API_ENDPOINT_PTZCONTROL,
                data={
                    "command": command,
                    "action": action,
                    "channelNo": "1",
                    "speed": speed,
                    "uuid": str(uuid4()),
                    "serial": serial,
                },
                headers={"clientType": "1"},
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        return req.text

    def login(self):
        """Set http session."""
        if self._session is None:
            self._session = requests.session()
            self._login()

        try:
            req = self._session.get(
                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_CHECKLOGIN,
                allow_redirects=False,
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError(
                "Could not access Ezviz login check API: " + str(err)
            ) from err

        if req.status_code != 200:
            # session is wrong, need to relogin
            self._login()

        else:
            response_json = req.json()
            if response_json["success"] != "success":
                self._login()

        return True

    def data_report(self, serial, enable=1, max_retries=0):
        """Enable alarm notifications."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        try:
            req = self._session.post(
                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_SET_DEFENCE,
                data={
                    "deviceSerial": serial,
                    "defenceType": "Global",
                    "enablePlan": enable,
                    "channelNo": "1",
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401 or req.status_code == 302:
            # session is wrong, need to relogin
            self.login()
            logging.info(
                "Got 302 or 401, relogging (max retries: %s)", str(max_retries)
            )
            return self.data_report(serial, enable, max_retries + 1)

        try:
            json_output = req.json()

        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        if json_output["resultCode"] != "0":
            raise PyEzvizError(
                f"Could not set the switch, maybe a permission issue ?: Got {req.status_code} : {req.text})"
            )

        return True

    def api_set_defence_schedule(self, serial, schedule, enable, max_retries=0):
        """Set defence schedules."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
        schedulestring = (
            '{"CN":0,"EL":'
            + str(enable)
            + ',"SS":"'
            + serial
            + '","WP":['
            + schedule
            + "]}]}"
        )
        try:
            req = self._session.post(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_SET_DEFENCE_SCHEDULE,
                data={
                    "serial": serial,
                    "csrfToken": self._csrf_token,
                    "devTimingPlan": schedulestring,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code != 200:
            # session is wrong, need to relogin
            self.login()
            logging.info("Error, got status code: %s", req.status_code)
            return self.api_set_defence_schedule(
                serial, schedule, enable, max_retries + 1
            )

        try:
            json_output = req.json()

        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        if json_output["resultCode"] != "0":
            raise PyEzvizError(
                f"Could not set the switch, maybe a permission issue ?: Got {req.status_code} : {req.text})"
            )

        return True

    def api_set_defence_mode(self, serial, mode: DefenseModeType, max_retries=0):
        """Set defence mode."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        try:
            req = self._session.post(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_SWITCH_DEFENCE_MODE,
                data={
                    "groupId": -1,
                    "mode": mode,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code != 200:
            # session is wrong, need to relogin
            self.login()
            logging.info("Error, got status code: %s", req.status_code)
            return self.api_set_defence_mode(serial, mode, max_retries + 1)

        try:
            json_output = req.json()

        except (OSError, json.decoder.JSONDecodeError) as err:
            raise PyEzvizError(
                "Impossible to decode response: "
                + str(err)
                + "\nResponse was: "
                + str(req.text)
            ) from err

        if json_output["meta"]["code"] != "200":
            raise PyEzvizError(
                f"Could not set the switch, maybe a permission issue ?: Got {req.status_code} : {req.text})"
            )

        return True

    def detection_sensibility(self, serial, sensibility=3, type_value=3, max_retries=0):
        """Set detection sensibility."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        if sensibility not in [0, 1, 2, 3, 4, 5, 6] and type_value == 0:
            raise PyEzvizError(
                "Unproper sensibility for type 0 (should be within 1 to 6)."
            )

        try:
            req = self._session.post(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_DETECTION_SENSIBILITY,
                data={
                    "subSerial": serial,
                    "type": type_value,
                    "channelNo": "1",
                    "value": sensibility,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401:
            # session is wrong, need to re-log-in
            self.login()
            logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
            return self.detection_sensibility(
                serial, sensibility, type_value, max_retries + 1
            )

        response_json = req.json()
        if response_json["resultCode"] and response_json["resultCode"] != "0":
            # raise PyEzvizError("Could not get detection sensibility: Got %s : %s)",str(req.status_code), str(req.text))
            return "Unknown error"

        return True

    def get_detection_sensibility(self, serial, type_value="0", max_retries=0):
        """Get detection sensibility notifications."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        try:
            req = self._session.post(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_DETECTION_SENSIBILITY_GET,
                data={
                    "subSerial": serial,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401:
            # session is wrong, need to re-log-in
            self.login()
            logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
            return self.get_detection_sensibility(serial, type_value, max_retries + 1)

        response_json = req.json()

        if response_json["resultCode"] != "0":
            return "Unknown"

        # Looks like 0 or 1 should return item 0
        if type_value == "1":
            return response_json["algorithmConfig"]["algorithmList"][0]["value"]

        for idx in response_json["algorithmConfig"]["algorithmList"]:
            if idx["type"] == type_value:
                return idx["value"]

        return "Unkown"

    # soundtype: 0 = normal, 1 = intensive, 2 = disabled ... don't ask me why...
    def alarm_sound(self, serial, sound_type, enable=1, max_retries=0):
        """Enable alarm sound by API."""
        if max_retries > MAX_RETRIES:
            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")

        if sound_type not in [0, 1, 2]:
            raise PyEzvizError(
                "Invalid sound_type, should be 0,1,2: " + str(sound_type)
            )

        try:
            req = self._session.put(
                "https://"
                + self.api_domain
                + API_BASE_TLD
                + API_ENDPOINT_DEVICES
                + serial
                + API_ENDPOINT_ALARM_SOUND,
                data={
                    "enable": enable,
                    "soundType": sound_type,
                    "voiceId": "0",
                    "deviceSerial": serial,
                },
                timeout=self._timeout,
            )

        except OSError as err:
            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err

        if req.status_code == 401:
            # session is wrong, need to re-log-in
            self.login()
            logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
            return self.alarm_sound(serial, sound_type, enable, max_retries + 1)
        if req.status_code != 200:
            logging.error("Got %s : %s)", str(req.status_code), str(req.text))

        return True

    def switch_status(self, serial, status_type, enable=0):
        """Switch status of a device."""
        return self._switch_status(serial, status_type, enable)

    def get_page_list(self):
        """Get ezviz device info broken down in sections."""
        return self._get_pagelist(
            page_filter="CLOUD, TIME_PLAN, CONNECTION, SWITCH, STATUS, WIFI, NODISTURB, KMS, P2P, TIME_PLAN, CHANNEL, VTM, DETECTOR, FEATURE, UPGRADE, VIDEO_QUALITY, QOS",
            json_key=None,
        )

    def get_device(self):
        """Get ezviz devices filter."""
        return self._get_pagelist(page_filter="CLOUD", json_key="deviceInfos")

    def get_connection(self):
        """Get ezviz connection infos filter."""
        return self._get_pagelist(page_filter="CONNECTION", json_key="connectionInfos")

    def get_status(self):
        """Get ezviz status infos filter."""
        return self._get_pagelist(page_filter="STATUS", json_key="statusInfos")

    def get_switch(self):
        """Get ezviz switch infos filter."""
        return self._get_pagelist(page_filter="SWITCH", json_key="switchStatusInfos")

    def get_wifi(self):
        """Get ezviz wifi infos filter."""
        return self._get_pagelist(page_filter="WIFI", json_key="wifiInfos")

    def get_nodisturb(self):
        """Get ezviz nodisturb infos filter."""
        return self._get_pagelist(
            page_filter="NODISTURB", json_key="alarmNodisturbInfos"
        )

    def get_p2p(self):
        """Get ezviz P2P infos filter."""
        return self._get_pagelist(page_filter="P2P", json_key="p2pInfos")

    def get_kms(self):
        """Get ezviz KMS infos filter."""
        return self._get_pagelist(page_filter="KMS", json_key="kmsInfos")

    def get_time_plan(self):
        """Get ezviz TIME_PLAN infos filter."""
        return self._get_pagelist(page_filter="TIME_PLAN", json_key="timePlanInfos")

    def close_session(self):
        """Close current session."""
        self._session.close()
        self._session = None
